package pl.pz.test.gps;

import java.util.List;

import android.content.Context;
import android.location.*;
import android.os.Bundle;


public class GpsReader
{
	//fields
	public final static float MINIMUM_ACCURACY = 100.0f;
	public final static Integer LOCATION_READY = 1;

	private final static String sProviderName = LocationManager.GPS_PROVIDER;		
	
	//fields
	protected Context context = null;
		
	private LocationManager locManager = null;	
	private LocationProvider locProvider = null;
	
	protected Boolean bStarted = false;
	
	
	//methods
	public GpsReader(Context context)
	{
		this.context = context;

		locManager = (LocationManager)context.getSystemService(Context.LOCATION_SERVICE);
		
		Main.log("Location providers:");
		final List<String> vecProviders = locManager.getAllProviders();
		for (String sName : vecProviders)
			Main.log("- " + sName);
		
		getLocationProvider();
	}
	
	private LocationProvider getLocationProvider()
	{
		//get location provider object
		locProvider = locManager.getProvider(sProviderName);
		
		return locProvider;
	}
		
	public boolean isEnabled()
	{
		if (locProvider == null)
			return false;
		if (sProviderName == null)
			return false;
		return locManager.isProviderEnabled(sProviderName);
	}
	
	public boolean isLocationProvider()
	{				
		return (locProvider != null);
	}		

	public boolean isStarted()
	{
		synchronized (bStarted)
		{
			return bStarted;
		}
	}
		
	//status event
	private GpsStatus.Listener eventGpsStatus = new GpsStatus.Listener()
	{
		public void onGpsStatusChanged(int event)
		{
			if (event == GpsStatus.GPS_EVENT_STARTED)
				Main.log("GPS_EVENT_STARTED");
			if (event == GpsStatus.GPS_EVENT_STOPPED)
				Main.log("GPS_EVENT_STOPPED");
			if (event == GpsStatus.GPS_EVENT_FIRST_FIX)
				Main.log("GPS_EVENT_FIRST_FIX");
		}		
	};
	
	//location event
	private LocationListener eventLocation = new LocationListener()
	{
		public void onLocationChanged(Location location)
		{	
			if (location == null)
				Main.log("location is null !");
			
			if (location != null)
			{				
				String sAlt = location.hasAltitude() ? "alt" : "";
				String sAccuracy = location.hasAccuracy() ? "accuracy" : "";

				Main.log("loc: " + sAlt + ", " + sAccuracy + " / " + Float.toString(location.getAccuracy()));
				
				final boolean bAccuracyAcceptable = isAccuracyAcceptable(location);

				if (bAccuracyAcceptable)
				{
					Main.log("Accuracy acceptable");
					stop();
				}
			}
		}

		public void onProviderDisabled(String provider)
		{
		}

		public void onProviderEnabled(String provider)
		{
		}

		public void onStatusChanged(String provider, int status, Bundle extras)
		{
			switch (status)
			{
				case LocationProvider.OUT_OF_SERVICE:
				{
					Main.log("OUT_OF_SERVICE");
					break;
				}
				case LocationProvider.TEMPORARILY_UNAVAILABLE:
				{
					Main.log("TEMPORARILY_UNAVAILABLE");
					break;
				}
				case LocationProvider.AVAILABLE:
				{
					Main.log("AVAILABLE");
					break;
				}
			}
		}
	};

		
	protected void start(int iUpdateDurationSeconds)	
	{		
		if (isStarted())
			return;
		
		if (!isLocationProvider())
			getLocationProvider();
		
		if (isEnabled())
		{
			synchronized (bStarted)
			{
				bStarted = false;
				try
				{
					locManager.requestLocationUpdates(sProviderName, (iUpdateDurationSeconds * 1000), 0, eventLocation);										
					locManager.addGpsStatusListener(eventGpsStatus);
					
				  Main.log("Started");
					
					bStarted = true;
				} catch (Exception e) {					
				}									
			}
		} else {			
		  Main.log("Service NOT enabled");
		}
	}	
	
	public void stop()
	{
		if (isStarted())
		{			
			synchronized (bStarted)
			{
				locManager.removeUpdates(eventLocation);
				locManager.removeGpsStatusListener(eventGpsStatus);
				
				bStarted = false;				
			}
		}
	}

	private boolean isAccuracyAcceptable(Location location)
	{
		if (location.hasAccuracy())
		{
			if (location.getAccuracy() < MINIMUM_ACCURACY)
			{
				return true;
			} else {
				return false;
			}			
		} else {
			return false;
		}
	}
	
}
